@@ -23,17 +23,22 @@
 #include "libaudcore/tuple.h"
 #include "libaudcore/vfs.h"
 
-typedef Tuple* pTuple;
+namespace audtag {
 
-typedef struct _module {
-    char *name;
-    int type; /* set to TAG_TYPE_NONE if the module cannot create new tags */
-    bool_t(*can_handle_file) (VFSFile *fd);
-    bool_t (* read_tag) (Tuple * tuple, VFSFile * handle);
-    bool_t (* read_image) (VFSFile * handle, void * * data, int64_t * size);
-    bool_t (* write_tag) (const Tuple * tuple, VFSFile * handle);
-} tag_module_t;
+struct TagModule {
+    const char *m_name;
+    TagType m_type; /* set to None if the module cannot create new tags */
 
-tag_module_t * find_tag_module (VFSFile * handle, int new_type);
+    virtual bool can_handle_file (VFSFile & file);
+    virtual bool read_tag (VFSFile & file, Tuple * tuple, Index<char> * image);
+    virtual bool write_tag (VFSFile & file, const Tuple & tuple);
+
+protected:
+    TagModule(const char *name, TagType type) : m_name(name), m_type(type) { };
+};
+
+TagModule * find_tag_module (VFSFile & handle, TagType new_type);
+
+}
 
 #endif /* TAG_MODULE_H */
